#!/usr/bin/perl -w
# 
# sb - Module for Serene Bach
# == written by T.Otani <ootani@segausers.gr.jp> ===
# == Copyright (C) 2004 SimpleBoxes/SerendipityNZ ==

package sb;

use strict;

# ==================================================
# // Module version
# ==================================================
use vars qw( $VERSION $WEBPAGE $PRODUCT @ISA );
$PRODUCT = 'Serene Bach';
$WEBPAGE = 'http://serennz.cool.ne.jp/sblog/';
$VERSION = '2.00b';
# 2.00a [2006/06/06] fixed a bug to change app correctly
# 2.00a [2005/05/12] added INIT_FILE as constant value
# 2.00a [2005/04/28] added DESTROY to clean up
# 2.00a [2005/03/02] added $PRODUCT, $WEBPAGE
# 2.00a [2005/02/05] added new as constructor to use stand-alone application
# 2.00a [2005/02/01] added check_module
# 2.00a [2004/11/17] generated ... oh, it's my birthday.

# ==================================================
# // configuration for inheritance / dependancy
# ==================================================
use sb::Config ();
use sb::Language ();
use sb::Interface ();
use sb::Plugin ();
use sb::Driver ();
use sb::App ();
# ==================================================
# // declaration for constant value
# ==================================================
sub INIT_FILE (){ 'init.cgi' }; # default config file is 'init.cgi'
# ==================================================
# // constructor
# ==================================================
sub new {
	my $class = shift;
	my $self = { 'config' => INIT_FILE, @_ };                # set config file
	my $conf = sb::Config->new('config'=>$self->{'config'}); # load config module
	sb::Language->new($conf->value('conf_lang'));            # load language module
	sb::Driver->new($conf->value('conf_dbtype'));            # load file driver
	sb::Plugin->new('dir'=>$conf->value('dir_plugin'));      # load plugin manager
	return bless($self,$class);
}
# ==================================================
# // destructor
# ==================================================
sub DESTROY { # cleaning up ...
	my $self = shift;
	sb::Interface->bye;
	sb::Plugin->bye;
	sb::Driver->bye;
	sb::Language->bye;
	sb::Config->bye;
	return();
}
# ==================================================
# // public functions - class method
# ==================================================
sub run { # running the application as CGI
	my $class = shift;
	my %param = ( 'app' => 'Main', @_ ); # default running module is 'Main'
	my $now   = time(); # store current GMT time
	my $app   = ref($class) ? $class : $class->new(%param);
	my $cgi   = sb::Interface->new('max_data'=>sb::Config->get->value('basic_max_data'));
	$app->check_module($cgi);
	eval { # main routine
		my $module = 'sb::App::' . $app->{'app'};
		eval("require $module;");
		$module->run('time'=>$now);
	};
	print sb::App->error($@) if ($@);
	return();
}
# ==================================================
# // public functions - instance method
# ==================================================
sub check_module { # module checker
	my $self = shift;
	my $cgi  = shift;
	if ($self->{'app'} eq 'Admin') {
		$self->{'app'} = 'Xmlrpc' if ($cgi->xmlflag);
	} elsif ($self->{'app'} eq 'Main') {
		if (sb::Config->get->value('basic_mobswitch') ne '') {
			my $ua_check = sb::Config->get->value('basic_mobswitch');
			$self->{'app'} = 'Mobile' if ($cgi->value('_agnt') =~ /$ua_check/);
		}
		if ($cgi->value('feed') ne '') {
			$self->{'app'} = 'Feed';
		}
	}
}
1;
__END__
