#!/usr/bin/perl -w
# 
# sb::App::Main - Module for Serene Bach
# == written by T.Otani <ootani@segausers.gr.jp> ===
# == Copyright (C) 2004 SimpleBoxes/SerendipityNZ ==

package sb::App::Main;

use strict;

# ==================================================
# // module version
# ==================================================
use vars qw( $VERSION @ISA );
$VERSION = '0.02';
# 0.02 [2005/07/11] changed config variable "srv_cgi" to "conf_srv_cgi"
# 0.01 [2005/07/08] changed run to set cookie correctly
# 0.00 [2005/02/01] generated

# ==================================================
# // configuration for inheritance / dependancy
# ==================================================
use sb::Interface ();
use sb::Config ();
use sb::Language ();
use sb::Data ();
use sb::Content ();
use sb::Receipt ();
use sb::App ();
@ISA = qw( sb::App );
# ==================================================
# // public functions
# ==================================================
sub run { # main routine
	my $class = shift;
	my $self = $class->SUPER::new( @_ );
	my $conf = sb::Config->get();
	my $cgi  = sb::Interface->get;
	my ($mode,$base,$css,$id,$cond) = &_check_mode($cgi,$conf);
	if ($mode eq 'tb' or $mode eq 'com') { # receive a trackback or a comment
		print sb::Receipt->new(
			'mode' => $mode,
			'cgi'  => $cgi,
			'id'   => $id,
			'time' => $self->{'time'},
		)->issue;
	} else { # view mode
		my $cookie = $cgi->cookie('name'=>$conf->value('basic_cooktag'));
		$cookie->{'checkid'} = $conf->value('conf_spamid');
		$cgi->set_cookie(
			'time'   => $self->{'time'},
			'name'   => $conf->value('basic_cooktag'),
			'expire' => 8544, # 8544 = 365 days [memo] set the same as sb::Receipt temporarily
			'data'   => $cookie,
		);
		my $type = ($mode eq 'css') ? 'text/css' : 'text/html';
		my $page = int( $cgi->value('page') );
		print $cgi->head('type'=>$type);
		print sb::Content->output($base,
			'mode' => $mode,
			'css'  => $css,
			'id'   => $id,
			'page' => $page,
			'cond' => $cond,
			'time' => $self->{'time'},
		);
	}
}
# ==================================================
# // private functions
# ==================================================
sub _check_mode { # checking mode
	my ($cgi,$conf) = @_;            # input parameters
	my ($mode,$base,$css,$id,$cond); # output parameters
	my $ping = $cgi->value('tb');
	if ($cgi->value('_path') ne '') {
		$ping = $cgi->value('_path');
		$ping =~ s/.*[:\/\\](.*)/$1/;
	}
	$ping = '' if ($ping !~ /^\d+$/);
	my $cid = $cgi->value('cid');
	my $tid = $cgi->value('tid');
	$mode = 'ent'  if ($cgi->value('eid') =~ /^\d+$/);      # entry mode
	$mode = 'user' if ($cgi->value('pid') =~ /^\d+$/);      # profile mode
	$mode = 'cat'  if ($cid =~ /^\d+$/);                    # category mode
	$mode = 'arc'  if ($cgi->value('month') =~ /^\d+$/);    # monthly archive mode
	$mode = 'arc'  if ($cgi->value('day') =~ /^\d+$/);      # daily archive mode
	$mode = 'srch' if ($cgi->value('search') ne '');        # search
	$mode = 'css'  if ($cgi->value('css') =~ /^\d+$/);      # style sheet
	$mode = 'tb'   if ($ping ne '');                        # receiving a trackback
	$mode = 'com'  if ($cgi->value('entry_id') =~ /^\d+$/); # receiving a comment
	$mode = 'page' if ($mode eq '');                        # page mode (default)
	if ($mode eq 'tb' or $mode eq 'com') {
		$id = ($mode eq 'tb') ? $ping : $cgi->value('entry_id');
		return($mode,$base,$css,$id,$cond);
	}
	my $flag = 0; # flag for changing template
	if ( $tid eq '' ) { # no specified template
		if ( $mode eq 'ent' ) { # entry mode
			my $entry = sb::Data->load('Entry','id'=>$cgi->value('eid'));
			die(sb::Language->get->string('error_no_entry') . "\n") if (!$entry);
			$cid = $entry->cat;
			if ($entry->stat == 0) {
				my $cookie = $cgi->cookie('name'=>$conf->value('basic_logtag'));
				if ($cookie->{'check'} ne $conf->value('basic_cookiekey')) {
					die(sb::Language->get->string('error_no_entry') . "\n");
				}
			}
		}
		if ( $cid ne '' ) { # category mode
			my $category = sb::Data->load('Category','id'=>$cid);
			$tid = $category->temp if ($category->temp > -1);
		}
		if ( $tid eq '' ) { # archive or profile mode
			$tid = $conf->value('conf_archive_temp') if ( $mode eq 'arc'  and $conf->value('conf_archive_temp') > -1);
			$tid = $conf->value('conf_profile_temp') if ( $mode eq 'user' and $conf->value('conf_profile_temp') > -1);
		}
		$tid = $cgi->value('css') if ( $mode eq 'css' );
		$flag = 1 if ($tid ne '');
	}
	my $temp = ($tid ne '') ? sb::Data->load('Template','id'=>$tid) : undef; # load template
	$temp = sb::Data->load('Template','id'=>sb::Data->load('Template','cond'=>{'use'=>1})->id) if (!$temp);
	die(sb::Language->get->string('error_unknown') . "\n") if (!$temp);
	# path of a css
	$css = $conf->value('conf_srv_base') . $conf->value('file_css');
	if ( $tid ne '' and (!$flag or $conf->value('conf_css_change')) ) {
		$css = $conf->value('conf_srv_cgi') . $conf->value('basic_sb') . '?css=' . $tid;
	}
	# set a base
	$base = ($mode eq 'ent' and $temp->entry ne '') ? $temp->entry : $temp->main if ($mode ne 'css');
	$base = $temp->css if ($mode eq 'css');
	# id for entry, user, or category
	$id = $cgi->value('eid') if ($mode eq 'ent');
	$id = $cgi->value('pid') if ($mode eq 'user');
	$id = $cgi->value('cid') if ($mode eq 'cat');
	# condition for search or archive
	$cond = ($cgi->value('month') ne '') ? $cgi->value('month') : $cgi->value('day') if ($mode eq 'arc');
	$cond = $cgi->value('search') if ($mode eq 'srch');
	return($mode,$base,$css,$id,$cond);
}
1;
__END__
