#!/usr/bin/perl -w
# 
# sb::Aws - Module for Serene Bach
# == written by T.Otani <ootani@segausers.gr.jp> ===
# == Copyright (C) 2004 SimpleBoxes/SerendipityNZ ==

package sb::Aws;

use strict;

# ==================================================
# // module version
# ==================================================
use vars qw( $VERSION @ISA );
$VERSION = '0.00';
# 0.00 [2005/04/12] port form sbaws.pl

# ==================================================
# // configuration for inheritance / dependancy
# ==================================================
use XML::Parser::Lite ();
use sb::Language ();
use sb::Text ();
use sb::App ();
@ISA = qw( sb::App );
# ==================================================
# // declaration for constant value
# ==================================================
sub AWS_SERVER        (){ 'http://xml-jp.amznxslt.com/onca/xml3?' };
sub AWS_ASSOCIATE     (){ 'simpleboxes-22' };
sub AWS_DEVELOPER     (){ 'D1XCY86MFV85YQ' };
sub AWS_LOCALE        (){ 'jp' };
sub AWS_TYPE          (){ 'lite' };
sub AWS_CODETYPE      (){ 'xml' };
sub AWS_ERROR_INITIAL (){ 'initialization error' };
sub AWS_ERROR_CONNECT (){ 'failed to connect' };
# ==================================================
# // declaration for class member
# ==================================================
my @mAwsData    = ();
my $mAwsCount   = 0;
my $mAwsMatched = 0;
my $mAwsPage    = 0;
my $mAwsTag     = undef;
my $mAwsErr     = undef;
# ==================================================
# // constructor
# ==================================================
sub new {
	my $class = shift;
	my $ua = $class->SUPER::init_agent;
	@mAwsData    = ();
	$mAwsCount   = 0;
	$mAwsMatched = 0;
	$mAwsPage    = 0;
	$mAwsTag     = undef;
	$mAwsErr     = undef;
	return bless( \$ua, $class );
}
# ==================================================
# // destructor
# ==================================================
sub DESTROY {
	my $self = shift;
	@mAwsData    = ();
	$mAwsCount   = 0;
	$mAwsMatched = 0;
	$mAwsPage    = 0;
	$mAwsTag     = undef;
	$mAwsErr     = undef;
	return();
}
# ==================================================
# // public functions
# ==================================================
sub get_data {
	my $self = shift;
	my $lang = sb::Language->get;
	my %param = (
		'dev-t'         => AWS_DEVELOPER,
		'type'          => AWS_TYPE,
		'f'             => AWS_CODETYPE,
		'locale'        => AWS_LOCALE,
		't'             => AWS_ASSOCIATE,
		'mode'          => undef,
		'KeywordSearch' => undef,
		'AsinSearch'    => undef,
		'page'          => 0,
		'charcode'      => 'utf8',
		@_
	);
	if ( !$$self ) {
		$mAwsErr = AWS_ERROR_INITIAL;
		return( undef );
	}
	my $text = undef;
	if ($param{'charcode'} eq 'utf8') {
		$text = $self->_connect(%param);
	} else {
		$lang->checkcode('',$param{'charcode'});
		$param{'KeywordSearch'} = $lang->convert($param{'KeywordSearch'},'utf8');
		$text = $self->_connect(%param);
		$lang->checkcode('','utf8');
		$text = $lang->convert($text,$param{'charcode'});
	}
	if ( !$text ) {
		$mAwsErr = AWS_ERROR_CONNECT;
		return( undef );
	} elsif ($text =~ /<ErrorMsg>(.*?)<\/ErrorMsg>/) {
		$mAwsErr = $1;
		return( undef );
	} else {
		my $parser = XML::Parser::Lite->new;
		$parser->setHandlers(
			Init  => \&_xml_init,
			Final => \&_xml_final,
			Start => \&_xml_start,
			Char  => \&_xml_char,
			End   => \&_xml_end,
		);
		$parser->parse($text);
		return( \@mAwsData );
	}
}
sub get_error {
	my $self = shift;
	return($mAwsErr);
}
sub get_count {
	my $self = shift;
	return($mAwsCount);
}
sub matched_number {
	my $self = shift;
	return($mAwsMatched);
}
sub total_pages {
	my $self = shift;
	return($mAwsPage);
}
# ==================================================
# // private functions
# ==================================================
sub _connect {
	my $self = shift;
	my %param = @_;
	my $url = AWS_SERVER;
	$param{'t'} = AWS_ASSOCIATE if ($param{'t'} eq '');
	foreach my $query ( keys(%param) ) {
		next if ($query eq 'charcode');
		next unless ( defined($param{$query}) );
		$url .= $query . '=' . &_encode_uri($param{$query}) . '&';
	}
	chop($url) if ($url =~ /\&$/);
	my $response = $$self->get($url);
	if ( index($response->{'_rc'},'2') == 0 ) {
		return $response->{'_content'};
	} else {
		return( undef );
	}
}
sub _encode_uri {
	$_[0] =~ s/(\W)/'%' . unpack('H2', $1)/eg;
	return($_[0]);
}
sub _xml_init {
	my $self = shift;
	# nothing to do
	return();
}
sub _xml_final {
	my $self = shift;
	# nothing to do
	return();
}
sub _xml_start {
	my $self = shift;
	my $tag = shift;
	if ($tag eq 'Details') {
		$mAwsData[$mAwsCount]->{'url'} = $_[1];
	} else {
		$mAwsTag = $tag;
	}
	return();
}
sub _xml_char {
	my $self = shift;
	my $elem = shift;
	if ($mAwsTag eq 'ProductName') { # product name
		$mAwsData[$mAwsCount]->{'name'} = $elem if ($elem ne '');
	} elsif ($mAwsTag eq 'Catalog') { # catalog
		$mAwsData[$mAwsCount]->{'cat'} = $elem if ($elem ne '');
	} elsif ($mAwsTag eq 'Artists' or $mAwsTag eq 'Authors') { # creators
		$mAwsData[$mAwsCount]->{'cre'} = []; # initialize 'cre' field
	} elsif ($mAwsTag eq 'Artist' or $mAwsTag eq 'Author') { # each creator
		push(@{$mAwsData[$mAwsCount]->{'cre'}},$elem) if ($elem ne '');
	} elsif ($mAwsTag eq 'ReleaseDate') { # release date
		$mAwsData[$mAwsCount]->{'days'} = $elem if ($elem ne '');
	} elsif ($mAwsTag eq 'Manufacturer') { # manufacturer
		$mAwsData[$mAwsCount]->{'make'} = $elem if ($elem ne '');
	} elsif ($mAwsTag eq 'ImageUrlSmall') { # small image
		$mAwsData[$mAwsCount]->{'ism'} = $elem if ($elem ne '');
	} elsif ($mAwsTag eq 'ImageUrlMedium') { # medium image
		$mAwsData[$mAwsCount]->{'imd'} = $elem if ($elem ne '');
	} elsif ($mAwsTag eq 'ImageUrlLarge') { # large image
		$mAwsData[$mAwsCount]->{'ilg'} = $elem if ($elem ne '');
	} elsif ($mAwsTag eq 'Availability') { # availability
		$mAwsData[$mAwsCount]->{'ava'} = $elem if ($elem ne '');
	} elsif ($mAwsTag eq 'ListPrice') { # list price
		$mAwsData[$mAwsCount]->{'lpr'} = $elem if ($elem ne '');
	} elsif ($mAwsTag eq 'OurPrice') { # their price
		$mAwsData[$mAwsCount]->{'opr'} = $elem if ($elem ne '');
	} elsif ($mAwsTag eq 'TotalResults') { # matched number of search
		$mAwsMatched = int($elem);
	} elsif ($mAwsTag eq 'TotalPages') { # total pages
		$mAwsPage = int($elem);
	}
	return();
}
sub _xml_end {
	my $self = shift;
	my $tag = shift;
	$mAwsCount++ if ($tag eq 'Details');
	$mAwsTag = undef;
	return();
}
1; # end of package
__END__
=head1 NAME

sb::Aws - Amazon Web Services Handler

=head1 SYNOPSIS

	require sb::Aws;
	my $aws = sb::Aws->new;
	my $result = $aws->get_data(
		't'             => 'webservices-20', # Amazon Associate ID
		'mode'          => 'category',       # Search Catgeory e.g. books-jp, books-us ....
		'KeywordSearch' => 'keyword',        # Search Keyword
		'AsinSearch'    => 'asin',           # Search ASID
		'page'          => 0,                # Display Page
		'charcode'      => 'utf8',           # Char Code e.g. utf8, euc, sjis ....
	);
	my $matched = $aws->matched_number; # get number of total results
	my $pages = $aws->total_page; # get number of the last page
	for (my $i=0;$i<$aws->get_count;$i++) {
		my $item = $result->[$i];
		# handles each item of results
	}

=head1 DESCRIPTION

sb::Aws handles serching on Amazon.co.jp via AWS.

sb::Aws::new works as constructor and returns LWP::UserAgent instance.

sb::Aws::get_data works as xml parser and returns search results of AWS as an array reference.
sb::Aws::get_data can be used following parameters
	't'             # Amazon Associate ID
	'mode'          # Search Catgeory e.g. books-jp, books-us ....
	'KeywordSearch' # Search Keyword
	'AsinSearch'    # Search ASID
	'page'          # Display Page
	'charcode'      # Output Char Code

sb::Aws::get_data returns an array reference.
Each element of array is a hash reference that has following data.
	'url'  # url for product
	'name' # product name
	'cat'  # catalog
	'cre'  # creators (stores as array)
	'days' # release date
	'make' # manufacturer
	'ism'  # url for small image
	'imd'  # url for medium image
	'ilg'  # url for large image
	'ava'  # availability
	'lpr'  # list price
	'opr'  # price on amazon

=head1 AUTHOR

Takuya Otani <mailto:ootani@segausers.gr.jp> http://serennz.cool.ne.jp/sb/

=head1 LICENSE

Copyright (C) 2004- Takuya Otani(SimpleBoxes) and SerendipityNZ

=cut
