#!/usr/bin/perl -w
# 
# sb::Data::Image - Module for Serene Bach
# == written by T.Otani <ootani@segausers.gr.jp> ===
# == Copyright (C) 2004 SimpleBoxes/SerendipityNZ ==

package sb::Data::Image;

use strict;

# ==================================================
# // module version
# ==================================================
use vars qw( $VERSION @ISA );
$VERSION = '0.00';
# 0.00 [2005/02/02] 

# ==================================================
# // configuration for inheritance / dependancy
# ==================================================
use sb::Config ();
use sb::Text ();
use sb::Data::Object ();
@ISA = qw( sb::Data::Object );
# ==================================================
# // declaration for constant value
# ==================================================
sub IMAGE_PREFIX     (){ 'img' };
sub THUMB_PREFIX     (){ 'thm' };
sub DEFAULT_FILENAME (){ 'file' };
sub PARTS_DIR        (){ '_parts/icon/' };
sub ICON_UNDEFINED   (){ 'undefined.gif' };
sub ICON_SUFFIX      (){ '.gif' };
sub ICON_WIDTH       (){ 32 };
sub ICON_HEIGHT      (){ 32 };
sub THUMB_CLASS      (){ 'thumb' };
sub IMAGE_CLASS      (){ 'pict' };
# ==================================================
# // declaration for data structure
# ==================================================
my %mStruct = (
	'content_list' => [
		'id',     # id
		'wid',    # wid
		'auth',   # 
		'date',   # 
		'name',   # ̾
		'file',   # ե̾
		'thumb',  # ͥ
		'stat',   # ᡼쥯ѥե饰
		'icon_c', # ȥѥե饰
		'icon_t', # ȥåХåѥե饰
		'dir',    # ¸
		'eid',    # ѵ
		'tz',     # ॾ
	],
);
# ==================================================
# // declaration for private variables
# ==================================================
my %pSizeFunction = (
	'.jpg' => \&_get_size_jpg,
	'.png' => \&_get_size_png,
	'.gif' => \&_get_size_gif,
);
# ==================================================
# // constructor
# ==================================================
sub alloc {
	my $class = shift;
	return $class->SUPER::alloc( @_, %mStruct );
}
# ==================================================
# // public functions
# ==================================================
sub dir {
	my $self = shift;
	$self->{'dir'} = shift if @_;
	return ($self->{'dir'} ne '') 
		? $self->{'dir'} 
		: sb::Config->get->value('conf_dir_img');
}
sub rename_file {
	my $self = shift;
	my %param = (
		'dir'   => $self->dir,
		'file'  => $self->file,
		'thumb' => $self->thumb,
		@_
	);
	if (  $param{'dir'} eq $self->dir and $param{'file'} eq $self->file and $param{'thumb'} eq $self->thumb ) {
		return( undef );
	} else {
		my $old_dir = sb::Config->get->value('conf_dir_base') . $self->dir;
		foreach my $type ('file','thumb') {
			next if ($self->$type() eq '');
			my $old = $old_dir . $self->$type();
			my $new = sb::Config->get->value('conf_dir_base') . $param{'dir'} . $param{$type};
			if ( rename($old,$new) ) {
				$self->$type($param{$type});
				$self->dir($param{'dir'});
			}
		}
		return( 1 );
	}
}
sub delete_file {
	my $self = shift;
	unlink($self->get_path);
	unlink($self->get_path('type'=>'thumb')) if ($self->thumb ne '');
}
sub erase {
	my $self = shift;
	$self->delete_file;
	$self->SUPER::erase;
}
sub upload {
	my $self = shift;
	my %param = (
		'entity' => undef,
		'label'  => undef,
		'dir'    => sb::Config->get->value('conf_dir_img'),
		'thumb'  => undef,
		'header' => [],
		'name'   => undef,
		'fixed'  => undef,
		'over'   => undef,
		@_
	);
	my $label = $param{'label'};
	my @header = @{$param{'header'}};
	return( undef ) if ($param{'entity'} eq '' or $label eq '' or !@header);
	foreach (@header) {
		next if ($_ !~ /$label/);
		$param{'type'}     = $2 if ($_ =~ /(.*)Content-type:(.*)/i);
		$param{'filename'} = $2 if ($_ =~ /(.*)filename=(.*)/i);
		$param{'macbin'}   = ($_ =~ /application\/x-macbinary/i) ? 1 : undef;
		last;
	}
	if ($param{'macbin'}) { # ޥåХʥ
		my $len = substr($param{'entity'},83,4);
		$len = unpack("%N",$len);
		$param{'entity'} = substr($param{'entity'},128,$len);
	}
	$param{'type'} =~ tr/\x0D\x0A//d;
	$param{'type'} =~ s/\s//g;
	$param{'filename'} =~ tr/\"\x0D\x0A//d;
	$param{'filename'} =~ s/.*[:\/\\](.*)/$1/; # ե̾
	my $tail = &_file_extension($param{'type'}) || &_check_tail($param{'filename'});
	my $name = ($param{'filename'} =~ /[^a-zA-Z0-9_\-\.]/)
	         ? IMAGE_PREFIX . $self->id . '_' . DEFAULT_FILENAME . $tail
	         : IMAGE_PREFIX . $self->id . '_' . $param{'filename'};
	$name = $param{'filename'} if ($param{'fixed'} and $param{'filename'} !~ /[^a-zA-Z0-9_\-\.]/);
	if ( $tail ) {
		$self->name(($param{'name'} ne '') ? sb::Text->entitize($param{'name'}) : sb::Text->entitize($param{'filename'}));
		$self->file($name);
		$self->dir($param{'dir'});
		my $file = $self->get_path;
		eval {
			die('file exit') if (!$param{'over'} and -e $file); # ե뤬Ǥ¸ߤƤ
			open(BINOUT,">$file") or die('failed file open'); # ե륪ץ
			binmode(BINOUT);
			print BINOUT $param{'entity'};
			close(BINOUT);
			chmod(sb::Config->get->value('basic_file_attr'),$file);
		};
		return( undef ) if ($@);
	} else {
		return( undef );
	}
	if ( $param{'thumb'} and &_resizable($self->file) ) {
		my $thumb = ($param{'filename'} =~ /[^a-zA-Z0-9_\-\.]/)
		          ? THUMB_PREFIX . $self->id . '_' . DEFAULT_FILENAME . $tail
		          : THUMB_PREFIX . $self->id . '_' . $param{'filename'};
		$thumb = $param{'filename'} if ($param{'fixed'} and $param{'filename'} !~ /[^a-zA-Z0-9_\-\.]/);
		my $base = sb::Config->get->value('conf_dir_base');
		my $size = sb::Config->get->value('conf_thumbsize');
		if ( &_create_smallimage($thumb,$name,$base . $self->dir,$size) ) {
			$self->thumb($thumb);
		}
	}
	return( 1 );
}
sub get_as_tag {
	my $self = shift;
	my %param = (
		'max_w' => 0,      # maximum width
		'max_h' => 0,      # maximum height
		'type'  => 'file', # file : ܲΤ / thumb : ͥΤ / link : ܲؤΥդ
		@_,
	);
	my ($w,$h) = $self->get_size(%param,'type'=>'file');
	if (&_resizable($self->file)) {
		if ($param{'type'} ne 'file' and $self->thumb ne '') { # ͥ뤢
			my ($tw,$th) = $self->get_size(%param,'type'=>'thumb');
			my $thumb = '<img src="' . $self->get_url('type'=>'thumb') . '" class="' . THUMB_CLASS . '" ';
			$thumb .= 'alt="' . $self->name . '" title="' . $self->name . '" ';
			$thumb .= 'width="' . $tw . '" height="' . $th . '" />';
			return ($param{'type'} eq 'thumb') ? $thumb : '<a href="' . $self->get_url . '">' . $thumb . '</a>';
		} else { # ͥʤ
			my $text = '<img src="' . $self->get_url . '" class="' . IMAGE_CLASS . '" ';
			$text .= 'alt="' . $self->name . '" title="' . $self->name . '" ';
			$text .= 'width="' . $w . '" height="' . $h . '" />';
			return $text;
		}
	} else {
		my $dir = sb::Config->get->value('dir_temp') . PARTS_DIR;
		my $srv = sb::Config->get->value('srv_temp') . PARTS_DIR;
		my $tail = &_check_tail($self->file);
		$tail =~ s/^\.//g;
		if ($tail ne '') {
			my $icon = $tail . ICON_SUFFIX;
			$icon = ICON_UNDEFINED if (!-e $dir . $icon);
			my $text = '<img src="' . $srv . $icon . '" class="' . IMAGE_CLASS . '" ';
			$text .= 'alt="' . $self->name . '" title="' . $self->name . '" ';
			$text .= 'width="' . ICON_WIDTH . '" height="' . ICON_HEIGHT . '" />';
			return ($param{'type'} ne 'link') ? $text : '<a href="' . $self->get_url . '">' . $text . '</a>';
		}
		return( undef );
	}
}
sub get_as_mime {
	my $self = shift;
	my %param = ( # ϥѥ᡼
		'type'  => 'file', # file or thumb
		@_,
	);
	my $out = '';
	return($out) if ($param{'type'} eq 'thumb' and $self->thumb eq '');
	my $file = $self->get_path('type'=>$param{'type'});
	my $len = sb::Config->get->value('basic_base_enc');
	if (-r $file) {
		require 'mimeutil.pl';
		open(IMAGEIN, $file);
		binmode(IMAGEIN);
		while( read(IMAGEIN, $_, $len) ) {
			$out .= &mimeutil::bodyencode($_,'b64');
		}
		$out .= &mimeutil::benflush('b64');
	}
	return($out);
}
sub is_image {
	my $self = shift;
	return &_resizable($self->file);
}
sub get_content_type {
	my $self = shift;
	return &_file_contenttype($self->file);
}
sub get_size {
	my $self = shift;
	my %param = ( # ϥѥ᡼
		'max_w' => 0,      # maximum width
		'max_h' => 0,      # maximum height
		'type'  => 'file', # file or thumb
		@_,
	);
	my ($w,$h) = (0,0); # ϥѥ᡼
	my $path = $self->get_path('type'=>$param{'type'});
	if (&_resizable($self->file)) {
		my $tail = &_check_tail($path);
		($w,$h) = &{$pSizeFunction{$tail}}($path) if ($tail);
		if ($param{'max_w'} and $param{'max_h'}) {
			return &_resize($w,$h,$param{'max_w'},$param{'max_h'});
		}
		return($w,$h);
	}
	return('--','--');
}
sub get_filesize {
	my $self = shift;
	my $path = $self->get_path('type'=>'file');
	my $size = (-s $path) / 1024; # KB 
	return sprintf("%.2f",$size);
}
sub get_path {
	my $self = shift;
	my %param = ( # ϥѥ᡼
		'type'  => 'file', # file or thumb
		@_,
	);
	my $dir  = sb::Config->get->value('conf_dir_base') . $self->dir;
	my $file = ($param{'type'} eq 'thumb' and $self->thumb ne '') ? $self->thumb : $self->file;
	return( $dir . $file );
}
sub get_url {
	my $self = shift;
	my %param = ( # ϥѥ᡼
		'type'  => 'file', # file or thumb
		@_,
	);
	my $dir  = sb::Config->get->value('conf_srv_base') . $self->dir;
	my $file = ($param{'type'} eq 'thumb' and $self->thumb ne '') ? $self->thumb : $self->file;
	return( $dir . $file );
}
sub initialize {
	my $self  = shift;
	my %param = @_;
	$param{'wid'} |= 0;
	$param{'auth'} |= 0;
	$param{'stat'} |= 0;
	$param{'icon_c'} |= 0;
	$param{'icon_t'} |= 0;
	$self->SUPER::initialize(%param);
}
# ==================================================
# // private functions
# ==================================================
sub _resizable {
	my $name = shift;
	my $type = &_file_contenttype($name);
	return ($type =~ /^image/);
}
sub _check_tail {
	my $file = shift;
	return &_file_extension(&_file_contenttype($file));
}
sub _file_extension {
	my $content_type = shift;
	return('.html') if ($content_type eq 'text/html');
	return('.css')  if ($content_type eq 'text/css');
	return('.txt')  if ($content_type eq 'text/plain');
	return('.xml')  if ($content_type eq 'application/xml');
	return('.jpg')  if ($content_type eq 'image/jpeg' or $content_type eq 'image/pjpeg');
	return('.png')  if ($content_type eq 'image/png' or $content_type eq 'image/x-png');
	return('.gif')  if ($content_type eq 'image/gif');
	return('.mov')  if ($content_type eq 'video/quicktime');
	return('.mpg')  if ($content_type eq 'vide/mpeg' or $content_type eq 'vide/x-mpeg');
	return('.avi')  if ($content_type eq 'vide/x-msvideo');
	return('.rm')   if ($content_type eq 'audio/x-pn-realaudio');
	return('.mp3')  if ($content_type eq 'audio/mpeg' or $content_type eq 'audio/x-mpeg');
	return('.midi') if ($content_type eq 'audio/midi' or $content_type eq 'audio/x-midi');
	return('.wav')  if ($content_type eq 'audio/wav' or $content_type eq 'audio/x-wav');
	return('.swf')  if ($content_type eq 'application/x-shockwave-flash');
	return('.gz')   if ($content_type eq 'application/gzip' or $content_type eq 'application/x-gzip');
	return('.zip')  if ($content_type eq 'application/zip' or $content_type eq 'application/x-zip-compressed');
	return('.lzh')  if ($content_type eq 'application/lha' or $content_type eq 'application/x-lha');
	return('.sit')  if ($content_type eq 'application/x-stuffit');
	return('.z')    if ($content_type eq 'application/x-compress');
	return( undef );
}
sub _file_contenttype {
	my $name = shift;
	return('text/html')                     if ($name =~ /\.html?$/i);
	return('text/css')                      if ($name =~ /\.css$/i);
	return('text/plain')                    if ($name =~ /\.txt$/i);
	return('image/gif')                     if ($name =~ /\.gif$/i);
	return('image/jpeg')                    if ($name =~ /\.jpe?g$/i);
	return('image/png')                     if ($name =~ /\.png$/i);
	return('audio/mpeg')                    if ($name =~ /\.mp3$/i);
	return('audio/midi')                    if ($name =~ /\.midi?$/i);
	return('audio/wav')                     if ($name =~ /\.wav$/i);
	return('audio/x-pn-realaudio')          if ($name =~ /\.rm$/i);
	return('video/mpeg')                    if ($name =~ /\.mpe?g$/i);
	return('video/x-msvideo')               if ($name =~ /\.avi$/i);
	return('video/quicktime')               if ($name =~ /\.mov$/i);
	return('application/xml')               if ($name =~ /\.xml$/i);
	return('application/x-shockwave-flash') if ($name =~ /\.swf$/i);
	return('application/x-stuffit')         if ($name =~ /\.sit$/i);
	return('application/x-compress')        if ($name =~ /\.z$/i);
	return('application/gzip')              if ($name =~ /\.gz$/i);
	return('application/zip')               if ($name =~ /\.zip$/i);
	return('application/lha')               if ($name =~ /\.lh[z|a]$/i);
	return('application/octet-stream');
}
sub _resize {
	my ($w,$h,$max_w,$max_h) = @_;
	if ($w > $max_w or $h > $max_h) { # ̾γ
		my $tmp = (($max_w / $w) < ($max_h / $h)) ? $max_w / $w : $max_h / $h;
		$w = int($w * $tmp) or 1;
		$h = int($h * $tmp) or 1;
	}
	return($w,$h);
}
sub _create_smallimage {
	my ($thumb,$file,$dir,$size) = @_;
	return( undef ) if ($file eq '' or $thumb eq '' or $dir eq '');
	eval {
		require Image::Magick;
		my $old = Image::Magick->new;
		$old->Read($dir . $file);
		my ($w,$h) = $old->Get('width','height');
		($w,$h) = &_resize($w,$h,$size,$size);
		my $new = $old->Clone();
		$new->Scale('width'=>$w,'height'=>$h);
		$new->Write("$dir$thumb");
		chmod(sb::Config->get->value('basic_file_attr'),$dir . $thumb) if (-e $dir . $thumb);
	};
	return ($@) ? undef : 1;
}
sub _get_size_jpg { # reference from KENT WEB
	my $jpeg = shift;
	my ($t, $m, $c, $l, $w, $h);
	open(JPEG, "$jpeg") or return (0,0);
	binmode JPEG;
	read(JPEG, $t, 2);
	while (1) {
		read(JPEG, $t, 4);
		($m, $c, $l) = unpack("a a n", $t);
		if ($m ne "\xFF") {
			$w = $h = 0;
			last;
		} elsif ((ord($c) >= 0xC0) && (ord($c) <= 0xC3)) {
			read(JPEG, $t, 5);
			($h, $w) = unpack("xnn", $t);
			last;
		} else {
			read(JPEG, $t, ($l - 2));
		}
	}
	close(JPEG);
	return($w, $h);
}
sub _get_size_gif { # reference from KENT WEB
	my $gif = shift;
	my ($data);
	open(GIF,"$gif") or return (0,0);
	binmode(GIF);
	sysread(GIF,$data,10);
	close(GIF);
	if ($data =~ /^GIF/) { $data = substr($data,-4); }
	my $w = unpack("v",substr($data,0,2));
	my $h = unpack("v",substr($data,2,2));
	return($w, $h);
}
sub _get_size_png { # WWWis (http://www.bloodyeck.com/wwwis/) 
	my $png = shift;
	my ($head, $a, $b, $c, $d, $e, $f, $g, $h);
	open(PNG,"$png") or return (0,0);
	binmode(PNG);
	if ( read(PNG, $head, 8) == 8 
	  && $head eq "\x89\x50\x4e\x47\x0d\x0a\x1a\x0a" 
	  && read(PNG, $head, 4) == 4 
	  && read(PNG, $head, 4) == 4 
	  && $head eq "IHDR" 
	  && read(PNG, $head, 8) == 8) {
		($a,$b,$c,$d,$e,$f,$g,$h) = unpack("C"x8, $head);
	} else {
		return(0,0);
	}
	close(PNG);
	return($a<<24|$b<<16|$c<<8|$d, $e<<24|$f<<16|$g<<8|$h);
}
1;
__END__
